@@ -12,9 +12,12 @@ import json
 class Background:
     def __init__(self,objname='None',gfile='None',tfile='None',extnum=1,
                  savefolder='EXTRA',overwrite=False,loadinfo=False,
+                 automask=True,
                  padxleft=5,padxright=5,
                  padyup=5,halfdyup=5,halfdylow=5,padylow=5,
-                 adjustx=0,adjusty=0
+                 adjustx=0,adjusty=0,
+                 rescale=(False,None,None,None),
+                 sigclip=(False,None,None)
                 ):
         # load instead of instantiate
         if loadinfo:
@@ -55,8 +58,13 @@ class Background:
                      'bbox':None
                      }
         self.bbox['bbox'] = self._bbox()
+        self.automask = automask
+        self.rescale = rescale
+        self.sigclip = sigclip
         self.bkg = self._bkg()
     def _bkg(self):
+        rescale = self.rescale
+        sigclip = self.sigclip
         tmpdata = fits.open(self.data['gfile'])[self.data['extnum']].data
         bb0x,bb1x = self.bbox['bbox']['BBX']
         bb0y,bb1y = self.bbox['bbox']['BBY']
@@ -66,22 +74,26 @@ class Background:
         x1 = np.arange(nx)
         x2 = np.arange(ny)
         x1v,x2v = np.meshgrid(x1,x2)
-        obj = Polynomial2D(x1v,x2v,tmpp)
+        obj = Polynomial2D(x1=x1v,x2=x2v,y=tmpp,rescale=rescale,sigclip=sigclip)
         obj.data['MASK'] = self._mask(obj)
         return obj
     def _mask(self,obj):
-        tmpmask = np.full_like(obj.data['Y'],False,dtype=bool)
-        halfdyup,halfdylow = self.bbox['halfdyup'],self.bbox['halfdylow']
-        xgn = (self.bbox['bbox']['XG'] - self.bbox['bbox']['BBX'][0]).astype(int)
-        ygn = (self.bbox['bbox']['YG'] - self.bbox['bbox']['BBY'][0]).astype(int)
-        minx = int(self.trace['XG'].min()) - self.bbox['bbox']['BBX'][0]
-        maxx = int(self.trace['XG'].max()) - self.bbox['bbox']['BBX'][0]
-        tracex = np.arange(minx,maxx+1)
-        for i,ii in enumerate(xgn):
-            if xgn[i] in tracex:
-                tmpx,tmpy = int(xgn[i]),int(ygn[i])
-                tmpmask[tmpy-halfdylow:tmpy+halfdyup+1,tmpx] = True
-        return tmpmask
+        if self.automask:
+            tmpmask = np.full_like(obj.data['Y'],False,dtype=bool)
+            halfdyup,halfdylow = self.bbox['halfdyup'],self.bbox['halfdylow']
+            xgn = (self.bbox['bbox']['XG'] - self.bbox['bbox']['BBX'][0]).astype(int)
+            ygn = (self.bbox['bbox']['YG'] - self.bbox['bbox']['BBY'][0]).astype(int)
+            minx = int(self.trace['XG'].min()) - self.bbox['bbox']['BBX'][0]
+            maxx = int(self.trace['XG'].max()) - self.bbox['bbox']['BBX'][0]
+            tracex = np.arange(minx,maxx+1)
+            for i,ii in enumerate(xgn):
+                if xgn[i] in tracex:
+                    tmpx,tmpy = int(xgn[i]),int(ygn[i])
+                    tmpmask[tmpy-halfdylow:tmpy+halfdyup+1,tmpx] = True
+            return tmpmask
+        else:
+            tmpmask = np.full_like(obj.data['Y'],False,dtype=bool)
+            return tmpmask
     def _rootname(self):
         tmp = fits.open(self.data['gfile'])[0].header['ROOTNAME']
         return tmp
@@ -101,7 +113,7 @@ class Background:
     def _bbox(self):
         xref,yref = self.trace['XREF'],self.trace['YREF']
         maxy,miny = self.trace['YG'].max(),self.trace['YG'].min()
-        bb0y,bb1y = int(miny-self.bbox['halfdylow']-self.bbox['padylow']),int(maxy+self.bbox['halfdyup']+self.bbox['padyup'])
+        bb0y,bb1y = int(miny-self.bbox['halfdylow']-self.bbox['padylow']),int(1+maxy+self.bbox['halfdyup']+self.bbox['padyup'])
         maxx,minx = self.trace['XG'].max(),self.trace['XG'].min()
         bb0x,bb1x = int(minx-self.bbox['padxleft']),int(maxx+self.bbox['padxright'])
         xg = np.arange(bb0x,bb1x)
@@ -301,14 +313,19 @@ class Background:
         yref = self.trace['YREF']
         tmp = self.bkg.data['Y'] - self.bkg.model['YFIT']
         tmpp = tmp.sum(axis=0)
+        tmppp = tmp.sum(axis=1)
         m = np.isfinite(tmp)
         vmin,vmax = np.percentile(tmp[m],minmax[0]),np.percentile(tmp[m],minmax[1])
         fig = plt.figure(figsize=figsize)
-        ax = fig.add_subplot(2,1,1)
+        ax = fig.add_subplot(2,2,1)
         ax.imshow(tmp,origin='lower',cmap=cmap,vmin=vmin,vmax=vmax)
         ax.set_title('{0}_bkgsub'.format(root),fontsize=fontsize)
         ax.set_ylabel('pixY - {0}'.format(int(yref)),fontsize=fontsize)
-        ax = fig.add_subplot(2,1,2,sharex=ax)
+        ax2 = fig.add_subplot(2,2,2,sharey=ax)
+        tmpx = np.arange(len(tmppp))
+        ax2.plot(tmppp,tmpx)
+        ax2.plot(np.full_like(tmppp,0.,dtype=float),tmpx,':')
+        ax = fig.add_subplot(2,2,3,sharex=ax)
         ax.plot(tmpp)
         tmpx = np.arange(tmpp.shape[0])
         tmpy = np.full_like(tmpx,annotate_level,dtype=float)
